#include <GL/glut.h>
#include <stdio.h>
#include <stdlib.h>

#ifndef _WIN32
#include <unistd.h>
#endif

#include "texturehandling.h"

/* Global ID-array */
unsigned int texture[TEX_NUM];
char *tex_names[] = {"textures/water1.bmp", "textures/water2.bmp", "textures/water3.bmp", "textures/water4.bmp", "textures/water5.bmp", "textures/grass1.bmp", "textures/grass2.bmp", "textures/grass3.bmp", "textures/grass4.bmp", "textures/grass5.bmp"};

/* This taken from NeHe, relatively unchanged */
int ImageLoad(char *filename, Image *image) {
	FILE *file;
	unsigned long size;
	unsigned long i;
	unsigned short int planes;
	unsigned short int bpp;
	char temp;

	/* make sure the file is there. */
	if ((file = fopen(filename, "rb"))==NULL)
	{
		printf("File Not Found : %s\n",filename);
		return 0;
	}

	/* seek through the bmp header, up to the width/height: */
	fseek(file, 18, SEEK_CUR);

	/* read the width */
	if ((i = fread(&image->sizeX, 4, 1, file)) != 1) {
		printf("Error reading width from %s.\n", filename);
		return 0;
	}

	/* read the height */
	if ((i = fread(&image->sizeY, 4, 1, file)) != 1) {
		printf("Error reading height from %s.\n", filename);
		return 0;
	}

	/* calculate the size (assuming 24 bits or 3 bytes per pixel). */
	size = image->sizeX * image->sizeY * 3;

	/* read the planes */
	if ((fread(&planes, 2, 1, file)) != 1) {
		printf("Error reading planes from %s.\n", filename);
		return 0;
	}
	if (planes != 1) {
		printf("Planes from %s is not 1: %u\n", filename, planes);
		return 0;
	}

	/* read the bpp */
	if ((i = fread(&bpp, 2, 1, file)) != 1) {
		printf("Error reading bpp from %s.\n", filename);
		return 0;
	}
	if (bpp != 24) {
		printf("Bpp from %s is not 24: %u\n", filename, bpp);
		return 0;
	}

	/* seek past the rest of the bitmap header. */
	fseek(file, 24, SEEK_CUR);

	/* read the data. */
	image->data = (char *) malloc(size);
	if (image->data == NULL) {
		printf("Error allocating memory for color-corrected image data");
		return 0;	
	}

	if ((i = fread(image->data, size, 1, file)) != 1) {
		printf("Error reading image data from %s.\n", filename);
		return 0;
	}

	/* reverse all of the colors. (bgr -> rgb) */
	for (i=0;i<size;i+=3) {
		temp = image->data[i];
		image->data[i] = image->data[i+2];
		image->data[i+2] = temp;
	}

	/* we're done. */
	return 1;
}

/* Load Bitmaps And Convert To Textures */
void LoadGLTextures() {	
	Image *image;
	int i;

	glGenTextures(TEX_NUM, texture);
	glGenTextures(TEX_NUM, texture);

	/* Load textures */
	for(i=0; i < TEX_NUM; i++) {
		image = (Image *) malloc(sizeof(Image));

		if (image == NULL) {
			printf("Error allocating space for image");
			exit(0);
		}

		if (!ImageLoad(tex_names[i], image)) {
			exit(1);
		}        

		/* Create Texture */
		glBindTexture(GL_TEXTURE_2D, texture[i]);

		/* scale linearly when image bigger than texture */
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		/* scale linearly when image smalled than texture */
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); 

		/* Enable repeating of texture */
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

		/* 2d texture, level of detail 0 (normal), 3 components (red, green, blue), x size from image, y size from image, 
		 * border 0 (normal), rgb color data, unsigned byte data, and finally the data itself. */
		glTexImage2D(GL_TEXTURE_2D, 0, 3, image->sizeX, image->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, image->data);
	}
}
